# Generated by Django 2.2.3 on 2019-08-22 09:26

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [("grapple", "0001_initial")]

    operations = [migrations.DeleteModel(name="PagePreview")]
